#!/usr/bin/env python3
#
# Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

import os
import re
import subprocess
import sys
import time

from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer


class GoHandler(FileSystemEventHandler):
    event_cache = {}

    def on_modified(self, event):
        seconds = int(time.time())
        key = (seconds, event.src_path)

        # check for identical events
        if key in self.event_cache:
            return

        # don't handle events if they happen quickly after another event
        if self.event_cache.__len__() != 0:
            previous_entry = list(self.event_cache)[-1]
            elapsed_time = seconds - previous_entry[0]
            print(f"elapsed_time between events: {elapsed_time}")
            if elapsed_time < 10:
                print(f"Too little time between events, Skipping...")
                return

        self.event_cache[key] = True

        # Only build pex if changes are made to files that end in ".py"
        if event.src_path.endswith(".go"):
            module = re.search("/go/(.*?)/", event.src_path).group(1)
            print(f"******************** Building module - {module} ********************")
            # run local-build-pexables.sh
            sp = subprocess.run(
                ["./bin/local-build-go.sh", f"{module}"],
                stdout=subprocess.PIPE,
                universal_newlines=True,
            )
            print(f"{sp.stdout}")
            print(f"*********************** {module} finished ***********************")
        print("Modification event handled.")


def build_missing_modules(path_to_sbin, path_to_modules):
    sbin_content = os.listdir(path_to_sbin)
    modules = os.listdir(path_to_modules)
    existing_modules_in_sbin = set(sbin_content).intersection(set(modules))
    if len(modules) > len(existing_modules_in_sbin):
        missing_modules = set(modules).difference(existing_modules_in_sbin)
        # print(
        #     f"There {'is' if len(missing_modules) == 1 else 'are'} "
        #     f"{len(missing_modules)} pex{'' if len(missing_modules) == 1 else 'es'} "
        #     f"missing from your sbin area: \n{missing_modules}"
        # )
        for mod in missing_modules:
            print(f"Building {mod}")
            sp = subprocess.run(
                ["./bin/local-build-go.sh", f"{mod}"],
                stdout=subprocess.PIPE,
                universal_newlines=True,
            )
            print(f"{sp.stdout}")
    else:
        print("There are no missing Go modules.")


if __name__ == "__main__":
    path_to_sbin = "/lustre/aoc/cluster/pipeline/docker/workspaces/sbin"
    path_to_modules = sys.argv[1] if len(sys.argv) > 1 else "./apps/cli/executables/go/"

    build_missing_modules(path_to_sbin, path_to_modules)
    print("Starting Go Watcher...")

    event_handler = GoHandler()
    observer = Observer()
    observer.schedule(event_handler, path_to_modules, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1)
    finally:
        observer.stop()
        observer.join()
